<?php include_once 'codebase/hyperframe.php' ?>
<h1>The Frame</h1>

<h3>constructing a frame for your site</h3>

<p>The <em>frame</em> consists of a directory containing a special webpage named <em>frame.php</em>, and various associated files. The function of the <em>frame</em> is similar to that of the <em>templates</em> used in mainstream CMS products, in that it determines the overall layout and appearance of all pages in the website. We use the term <em>frame</em> because the use of template in this context is actually a misnomer. Hyperframe also has templates, and these serve -in the correct sense of the word- as outlines on which to base new pages.</p>

<p>In Hyperframe, any changes to the frame will profoundly affect <em>all</em> existing site pages, and <em>all</em> new pages. A change to a template will only affect <em>new</em> pages based on that template, created <em>after</em> the template was modified.&nbsp;</p>

<p>One of the key tasks for any developer setting-up a Hyperframe site will be to construct a <em>frame</em> for that site. This task is somewhat more complex than the creation of a typical webpage, but not drastically so. It only needs to be done once, per site. Because of the special notation used in some places, it is best done in a text editor, and not with the online tools.&nbsp;</p>

<p>Note that we are not talking about Netscape frames. Hyperframe sites do not use the HTML frameset tag. I guess the similarity of naming&nbsp; might lead to some confusion, or even misapprehension by standards purists here, but I feel that the frameset tag is so seldom used these days that it is acceptable to reuse the term.</p>

<p>To see what a frame contains, navigate to the frame/ directory of this sample site and examine its contents. The main component of each frame is frame.php. Incidentally this file is always named <em>frame.php</em>, but the directory holding it can in principle have any name, allowing for multiple frames to exist.&nbsp;&nbsp;</p>

<h3>The frame.php file</h3>

<p>This is a standard php/HTML file, of which only the &lt;head&gt; and &lt;body&gt; sections are used by the system. <em>The rest is ignored</em>.</p>

<h4>A typical &lt;head&gt; section might contain:</h4>

<pre>
&lt;head&gt;
  &lt;title&gt;Hyperframe !CMS&lt;/title&gt;
  &lt;meta name=&quot;description&quot; content=&quot;HTML pages, content managed&quot;&gt;
  &lt;meta name=&quot;keywords&quot; content=&quot;content management, web frontend, html, pages&quot;&gt;
  &lt;META NAME=&quot;hf_doctype&quot; CONTENT=&quot;HTML&quot;&gt;
  &lt;META NAME=&quot;hf_hide&quot; CONTENT=&quot;&quot;&gt;
  &lt;meta name=&quot;Author&quot; content=&quot;IWR Consultancy&quot;&gt;
  &lt;meta http-equiv=&quot;Content-Type&quot; content=&quot;text/html; charset=utf-8&quot;&gt;
  &lt;link rel=stylesheet href=&quot;&lt;?php echo codedir ?&gt;system.css&quot; type=&quot;text/css&quot;&gt;
  &lt;link rel=stylesheet href=&quot;&lt;?php echo framedir ?&gt;frame.css&quot; type=&quot;text/css&quot;&gt;
  &lt;script language=javascript src=&quot;&lt;?php echo codedir ?&gt;system.js&quot;&gt;&lt;/script&gt;
  &lt;script language=javascript src=&quot;&lt;?php echo framedir ?&gt;frame.js&quot;&gt;&lt;/script&gt;
  <strong>&lt;?php include_once plugin(&#39;menu&#39;); ?&gt;</strong>
&lt;/head&gt;</pre>

<p><em>The frame&#39;s head section in fact forms the definitive head section of all pages</em>, with the exception that certain items in the calling page&#39;s own head section may modify or over-ride those in the frame.</p>

<p>An important addition in Hyperframe 4 is the call to the menu plugin (bolded) which must be present&nbsp; if the cascading menu plugin is used in pages. (If you use your own menu code you can omit this)</p>

<p>The <strong>title</strong> is used on all pages, and is prepended to any title found on an individual page. This conveniently avoids the need to keep repeating the site&#39;s name in titles. Where a page contains no title. the frame title is used, with the first &lt;Hn&gt; tag&#39;s contents appended. This in most cases will give rise to an acceptable title string which characterises the page&#39;s content.</p>

<p>The frame&#39;s <strong>description</strong> meta tag is only used if the calling page contains no <strong>title</strong>. If the page contains a title, its own description is used.</p>

<p>The <strong>author</strong> content -and any other unspecified meta content- applies to all pages, regardless.</p>

<h4>Several special hf_xxxx meta tags exist:</h4>

<p><strong>hf_doctype</strong> determines the &lt;!doctype&gt; declaration placed at the top of each rendered page. The possible values are:&nbsp;<br />
HTML (Default, HTML5)<br />
HTML5<br />
HTML4<br />
HTML4T<br />
HTML4F<br />
-which should be self-explanatory.<br />
To reiterate, any doctype declaration in the first line of the frame <em>will be ignored,</em> as will any in the calling page. The reason it&#39;s done like this, by the way, is that the online editor needs to know the document type, and it&#39;s simpler to use a variable of a known format than to scan the file for it. It also means that any extraneous doctype tags on imported pages will not affect operation.</p>

<p>Hyperframe does not support XHTML. This is not the problem it might seem for imported pages, since the majority of XHTML documents will display perfectly well as HTML5.</p>

<p>HTML4 support, which is mainly provided for easy import of legacy sites, includes switching the editor&#39;s semantics to generate the tags preferred by that standard. For example in HTML4 mode, the Bold&nbsp; button will insert a&nbsp; &lt;b&gt; tag, in HTML5 mode it will insert a &lt;strong&gt; tag.</p>

<p><strong>hf_hide</strong> takes any of the values <em>banner, top, left, content, right, bottom</em>&nbsp; comma separated, or the keyword <em>all</em>. It causes the relevant hf-[section] ID to <em>not</em> be displayed.&nbsp; hf_hide can be used in the frame, or in the calling page. This is useful where pages contain very wide content, and need more space.</p>

<h4>The frame body</h4>

<p>The body section of frame.php defines the master layout of all page using this frame.</p>

<p>In this example from the Slate frame, we have a top banner, a top dropdown menu and a fixed lefthand bar containing an image. There is no righthand bar. Divs provide layout.&nbsp; Although it&#39;s not apparent here, if you examine the associated frame.css file you will see that this frame creates fixed-width pages.</p>

<p>Since only one menu is used, the menu filename is given directly as <em>&#39;tree.mnu</em>&#39;&nbsp; -which file will be in the <em>sitecfg</em> directory. The <em>dropdown </em>attribute sets the behaviour to be that of horizontal menu bar, whilst the <em>menutree</em> and <em>topmenu</em> classes of the menu container determine the css sections (in frame.css) needed to make this work.&nbsp;</p>

<pre>
&lt;body&gt;
&lt;div class=&quot;hf_pagecontainer&quot;&gt;
  &lt;div id=&quot;hf_banner&quot; &gt;
    &lt;h1&gt;Hyperframe flatfile CMS&lt;/h1&gt;
  &lt;/div&gt;
  &lt;div id=&quot;hf_top&quot; &gt;
    &lt;div class=&quot;menutree topmenu&quot; &gt;
    &lt;a class=&quot;menu_single&quot; href=&quot;&lt;?php echo siteroot ?&gt;sitemap.php&quot;&gt;Site Map&lt;/a&gt;
    &lt;?php menu(&#39;tree.mnu&#39;,&#39;dropdown&#39;);?&gt;&lt;/div&gt;
    &lt;div style=&#39;text-align:right&#39;&gt;&lt;?php widgets();?&gt;&amp;nbsp;&lt;/div&gt;
  &lt;/div&gt;
  &lt;div id=&quot;midsection&quot;&gt;
    &lt;div id=&quot;hf_left&quot; &gt;
     &lt;img src=&quot;&lt;?php echo framedir ?&gt;sidelogo.gif&quot;&gt;
    &lt;/div&gt;
    &lt;div id=&quot;hf_content&quot; &gt;
      &lt;!--CONTENT--&gt;
    &lt;/div&gt;
  &lt;/div&gt;
  &lt;div id=&quot;hf_bottom&quot; &gt;
   Powered by Hyperframe &lt;br&gt;
  &lt;/div&gt;
  &amp;nbsp;  
&lt;/div&gt;
&lt;/body&gt;

</pre>

<p>Another example, this time from the Tabula frame. This uses a three-column layout, although the righthand bar is presently empty save for a spacer. Layout is, as the name suggests, by way of the more traditional table method. If you examine the associated frame.css&nbsp; you will see that it&#39;s a variable-width page which adjusts its size to suit the browser window.&nbsp; A max width of 1400px is set, though, to prevent excessively long text lines when displayed on large screens. Where this width is exceeded, the content will automatically center itself in the browser window.</p>

<p>In this frame two menus are used, a smaller top &#39;quicklink&#39; menu with the most often-used links, and a cascading side menu with all the links. To achieve this, the sitemenu function is called, as opposed to invoking the menus directly. When done this way,&nbsp; opening a page with the &#39;?menu=top&#39; parameter will result in the top menu being the main one -which can be useful on fullwidth pages with&nbsp; no sidebar.</p>

<pre>
&lt;body&gt;
&lt;table class=&quot;hf_pagelayout&quot;&gt;
&lt;tr&gt;&lt;td id=&quot;hf_banner&quot; &gt;
  &lt;div float=&#39;left&#39; align=&#39;left&#39;&gt;&lt;img src=&#39;&lt;?php echo framedir ?&gt;hylogo.png&#39; align=&quot;left&quot;&gt;&lt;/div&gt;
  &lt;div float=&#39;right&#39; align=&#39;right&#39;&gt;&lt;span class=&#39;callout&#39;&gt;by IWR Consultancy&lt;/span&gt;&lt;/div&gt;
&lt;/td&gt;&lt;/tr&gt;
&lt;tr&gt;&lt;td id=&quot;hf_top&quot;&gt;
  &lt;div float=&#39;left&#39; align=&#39;left&#39;&gt;&lt;?php sitemenu(&#39;top&#39;);?&gt;&lt;/div&gt;
  &lt;div float=&#39;right&#39; align=&#39;right&#39;&gt;&lt;?php printbutton();?&gt;&amp;nbsp;&lt;/div&gt;
&lt;/td&gt;&lt;/tr&gt;&lt;/table&gt;

&lt;table class=&quot;hf_pagelayout&quot;&gt;&lt;tr&gt;
&lt;td id=&quot;hf_left&quot;&gt;
  &lt;?php sitemenu(&#39;side&#39;)?&gt;
  &lt;hr width=90%&gt;
&lt;/td&gt;
&lt;td id=&quot;hf_content&quot; &gt;
  &lt;!--CONTENT--&gt;
&lt;/td&gt;
&lt;td id=&quot;hf_right&quot;&gt;
  &amp;nbsp;
&lt;/td&gt;&lt;/tr&gt;&lt;/table&gt;

&lt;table class=&quot;hf_pagelayout&quot;&gt;
&lt;tr id=&quot;hf_bottom&quot;&gt;&lt;td &gt;
  &lt;p class=&#39;callout&#39;&gt;Powered by Hyperframe&lt;/p&gt;
&lt;td&gt;&lt;/tr&gt;&lt;/table&gt;
&lt;/body&gt;</pre>

<p>In all frames, the structure is that of a set of regions bearing these css IDs:<br />
<strong>ID: hf_banner</strong> - a top banner<br />
<strong>ID: hf_top</strong> - a topmenu line<br />
<strong>ID: hf_left</strong> - a lefthand bar<br />
<strong>ID: hf_content</strong>&nbsp; - the main content area<br />
<strong>ID: hf_right</strong> - a righthand bar (if used)<br />
<strong>ID: hf_bottom</strong> - a footer area.</p>

<p>Because these css ID&#39;s are referred-to internally by Hyperframe, they should always be present in any frame <em>if</em> the relevant area exists. You can use these predetermined IDs for styling purposes, or you can add as many IDs or classes as you like of your own.&nbsp; Layout can be built using divs, tables or HTML5 document section attributes as preferred.</p>

<p>By the way, there&#39;s no doubt that the mere suggestion of using a table for layout will get the soapboxers ranting, but it&#39;s a fact that layout using divs is a much harder art to master, and that a trivial mistake in div-based layout is very likely to leave you with a styling disaster like overlapping text. So, we&#39;re not making any hard and fast rule on that one. Do what you are comfortable with, and remember that in the end it&#39;s a site which displays correctly, and not as a jumbled mess or as overlapping text, that matters. If the site text is unreadable, then compliance with the standards textbook is irrelevant. The main point here, I think, is that layout using <em>nested</em> tables is very bad practice, and is almost never necessary.</p>

<p>If you look at the Divvy frame, you will see an example of div-based layout but with table-like behaviour, which is possible in the latest release of css. In some ways this is the optimal solution, offering the predictable behaviour of tables whilst also complying with the latest standards. It is somewhat more verbose and complicated than either of the conventional methods, though. Bear in mind this won&#39;t work in older browsers, either.</p>

<h4>Images in the frame:</h4>

<p>A point to note is that images or media are loaded from the <em>calling page&#39;s</em> context, not the frame&#39;s context. Thus, images to be incorporated into the frame need to be either css backgrounds, or else need to be called with the special syntax : src=&#39;<strong>&lt;?php echo framedir ?&gt;</strong>filename&#39; -which will load the image from the frame directory regardless of what that is called-&nbsp; which is usually what is wanted.</p>

<p>The hf_content area contains a special tag, <strong>&lt;!--CONTENT--&gt;</strong> which must be <em>exactly</em> as shown. This is not a comment, but a placeholder for the individual page content within the frame. So far this is the only example of &#39;duck code&#39; within Hyperframe, and I&#39;d rather it were not there.. but this proved to be the simplest approach, so that&#39;s the way it is.</p>

<p>A frame has a subdirectory, <strong>menu_img</strong> which contains the small placeholder icons used in the lefthand cascading menu. Since it might be desirable to customise these for specific frames, it makes more sense for them to be bundled with the frame rather than with the menu itself.</p>

<h4>The frame.css file:</h4>

<p>The place you will likely do most customising is the frame stylesheet, <strong>frame.css</strong>. This has four sections: Global styles which apply to standard tags, styles which apply to the predetermined IDs mentioned above, styles applying to the lefthand menu, and styles applying to the top menu. Note that whilst you can do more or less what you like with the others, some elements of the the <em>top menu</em> styling are essential to its function, so test any changes here carefully.&nbsp;</p>

<h4>Automation:</h4>

<p>The frame can contain any php or javascript code. A point to note if including php is that the execution order is system php code, php code in the calling page, <em>then</em> php code in the frame. Thus, values changed in the frame have no effect on those settings as far as the calling page is concerned. <em>This is probably the reverse of what you would intuitively expect, so take note. </em>Since javascript is executed by the browser after the page has been sent, this consideration does not apply there.</p>
